// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "CharacterController2D.hpp"

using namespace Dawn;

CharacterController2D::CharacterController2D(SceneItem *i) :
  SceneItemComponent(i)
{
}

void CharacterController2D::onStart() {
  useEvent([&](float_t delta){
    // Friction
    velocity -= velocity * friction * delta;

    auto myCollider = item->getComponent<Collider2D>();
    glm::vec2 currentPosition(
      this->transform->getLocalPosition().x,
      this->transform->getLocalPosition().z
    );


    // Perform movement and check for collisions.
    glm::vec2 moveAmount;
    if(myCollider == nullptr) {
      moveAmount = velocity;
    } else {
      // Perform sweep
      auto allColliders = getScene()->findComponents<Collider2D>();
      auto itColliders = allColliders.begin();

      glm::vec2 normal;
      float_t entryTime;
      float_t exitTime;
      glm::vec2 entryPoint;
      glm::vec2 exitPoint;
      bool_t result;

      // Check for collisions, definitely not working 100% yet 
      while(itColliders != allColliders.end()) {
        auto c = *itColliders;
        ++itColliders;
        if(c == myCollider) continue;
        result = myCollider->getCollidingResult(
          velocity, c, normal, entryTime, exitTime, entryPoint, exitPoint
        );
        if(result) break;
      }

      if(result && entryTime <= delta) {
        // Slide Resolution https://www.gamedev.net/articles/programming/general-and-gameplay-programming/swept-aabb-collision-detection-and-response-r3084/
        float dotprod = (velocity.x * normal.y) + (velocity.y * normal.x) * (1.0f - entryTime);
        moveAmount = velocity * entryTime;

        // Respond to changes
        velocity.x = dotprod * normal.y; 
        velocity.y = dotprod * normal.x;
      } else {
        moveAmount = velocity;
      }
    }

    transform->setLocalPosition(
      transform->getLocalPosition() + (glm::vec3(moveAmount.x, 0, moveAmount.y) * delta)
    );
  }, getScene()->eventSceneUpdate);
}