// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "UIComponent.hpp"
#include "display/font/Font.hpp"

#define UI_LABEL_MAX_WIDTH_NONE -1
#define UI_LABEL_MAX_WIDTH_ALIGN 0

namespace Dawn {
  class UILabel : public UIComponent, public UIComponentRenderable {
    private:
      bool_t needsRebuffering = true;
      Mesh mesh;

      /**
       * Returns true if the label contains renderable text.
       * 
       * @return True if the label has text, otherwise false.
       */
      bool_t hasText();

      /**
       * Internally performs the mesh update.
       */
      void updateMesh();

    public:
      StateProperty<std::string> text;
      StateProperty<float_t> fontSize;
      StateProperty<Font*> font;
      StateProperty<float_t> maxWidth;
      struct Color textColor = COLOR_WHITE;
      struct FontMeasure measure;
      int32_t startQuad = 0;
      int32_t quadCount = -1;

      UILabel(SceneItem *item);

      float_t getContentWidth() override;
      float_t getContentHeight() override;
      std::vector<struct ShaderPassItem> getPassItems(
        glm::mat4 projection, glm::mat4 view
      ) override;
      void onStart() override;
  };
}