// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"
#include "scene/components/PlayerController.hpp"
#include "scene/components/GameCamera.hpp"
#include "scene/components/GameCamera.hpp"
#include "scene/components/display/MeshRenderer.hpp"
#include "scene/components/display/MeshHost.hpp"
#include "scene/components/display/material/SimpleTexturedMaterial.hpp"
#include "scene/components/example/ExampleSpin.hpp"
#include "scene/components/physics/2d/BoxCollider.hpp"
#include "scene/components/physics/2d/CharacterController2D.hpp"
#include "display/mesh/CubeMesh.hpp"

namespace Dawn {
  class HelloWorldScene : public Scene {
    protected:
      Camera *camera;

      void stage() override {
        auto playerItem = this->createSceneItem();
        auto player = playerItem->addComponent<PlayerController>();
        auto hitbox = playerItem->addComponent<BoxCollider>();
        playerItem->addComponent<CharacterController2D>();

        auto wallItem = this->createSceneItem();
        auto wallBox = wallItem->addComponent<BoxCollider>();
        wallBox->min = glm::vec2(-4, -3);
        wallBox->max = glm::vec2(-3, 3);
        
        camera = Camera::create(this);
        camera->transform->lookAt(glm::vec3(10, 10, 10), glm::vec3(0, 0, 0));
        // auto gameCamera = camera->item->addComponent<GameCamera>();
        // gameCamera->player = player;
      }
      
      std::vector<Asset*> getRequiredAssets() override {
        auto assMan = &this->game->assetManager;
        std::vector<Asset*> assets;
        return assets;
      }

    public:
      HelloWorldScene(DawnGame *game) : Scene(game) {}
  };
}